
library(ape)
library(BAMMtools)

v <- read.tree("data/actinopt_12k_treePL.tre")
v <- check_and_fix_ultrametric(v)

xx <- read.csv("data/marine-fw-rates-june2018.csv", stringsAsFactors=F)
rownames(xx) <- xx$treename

#----- State scoring
# categories
#   Fw
#   Fw-Ma
#   Ma
#   Br-Ma
#   Br-Fw
#   Br-Fw-Ma
#   Br

salinity <- character(nrow(xx))
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 0] <- "Fw"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Fw-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 0] <- "Fw-Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 0] <- "Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 0 & xx$Brackish == 1] <- "Br"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 1] <- "Br-Fw"

xx$salinity <- salinity


#------------------------------
# Consider only species with genetic data:

xx <- xx[ rownames(xx) %in% v$tip.label, ]

v <- drop.tip(v, setdiff(v$tip.label, rownames(xx)))
 

#--------------------------------------
# Model of character evolution
#   
#   rate: marine to brackish
#   rate: fw to brackish
#   rate: brackish-marine to FW
#   rate: fw-brackish to marine
#   rate: loss of state (equal across all)

library(diversitree) 

xx <- xx[v$tip.label, ] 
 
statevec  <- rep(NA, nrow(xx)) 
statevec[xx$salinity == "Fw"]           <- 1
statevec[xx$salinity == "Ma"]           <- 2
statevec[xx$salinity == "Br"]           <- 3
statevec[xx$salinity == "Br-Fw"]        <- 4
statevec[xx$salinity == "Br-Ma"]        <- 5
statevec[xx$salinity == "Br-Fw-Ma"]     <- 6
 
names(statevec) <- rownames(xx)
 
lfx <- make.mkn(v, states= statevec, k=6) 

#f_constraints  <- list( q12 = 0, q13 = 0 , q14 = XXX, q15 = 0, q16 = 0,
#					    q21 = 0 , q23 = 0, q24 = 0, q25 = XXX, q26 = 0, 
#					    q31 = 0, q32 = 0, q34 = XXX, q35 = XXX, q36 = 0, 
#					    q41 = XXX, q42 = 0, q43 = XXX, q45 = 0, q46 = XXX , 
#					    q51 = 0, q52 = XXX , q53 = XXX, q54 = 0, q56 = XXX , 
#					    q61 = 0, q62 = 0, q63 = 0, q64 = XXX, q65 = XXX)
 
# pars: 
#         q14 : Fw    -> Br-Fw
#         q25 : Ma    -> Br-Ma
#         q34 : Br    -> Br-Fw
#         q35 : Br    -> Br-Ma
#         q41 : Br-Fw -> Fw        
#         q43 : Br-Fw -> Br                         = q41
#         q46 : Br-Fw -> Br-Fw-Ma  = (Br -> Br-Ma ) = q35
#         q52 : Br-Ma -> Ma                         = q41
#         q53 : Br-Ma -> Br                         = q41
#         q56 : Br-Ma -> Br-Fw-Ma  = (Br -> Br-Fw)  = q34
#         q64 : Br-Fw-Ma -> Br-Fw                   = q41
#         q65 : Br-Fw-Ma -> Br-Ma                   = q41 
 
f_constraints  <- list( q12 ~ 0, q13 ~ 0,  q15 ~ 0, q16 ~ 0,
					    q21 ~ 0 , q23 ~ 0, q24 ~ 0, q26 ~ 0, 
					    q31 ~ 0, q32 ~ 0, q36 ~ 0, 
					    q42 ~ 0, q43 ~ q41, q45 ~ 0, q46 ~ q35 , 
					    q51 ~ 0, q52 ~ q41 , q53 ~ q41, q54 ~ 0, q56 ~ q34 , 
					    q61 ~ 0, q62 ~ 0, q63 ~ 0, q64 ~ q41, q65 ~ q41)
					    
lfx_con <- constrain(lfx, formulae = f_constraints)
 
partable <- function(x){
	
	pset <- exp(x)
	#  in order: q14, q25, q34, q35, q41
	
	
	rate       <- numeric(12)
	code       <- character(12)
	code_con   <- character(12)
	transition <- character(12)
	
	code     <- c("q14", "q25", "q34", "q35", "q41", "q43", 
				"q46", "q52", "q53", "q56", "q64", "q65")
	
	code_con <- c("q14", "q25", "q34", "q35", "q41", "q41", 
				"q35", "q41", "q41", "q34", "q41", "q41")	
	
	transition <- c("Fw -> Br-Fw", "Ma -> Br-Ma", "Br -> Br-Fw", "Br -> Br-Ma",
	                 "Br-Fw -> Fw", "Br-Fw -> Br", "Br-Fw -> Br-Fw-Ma", 
	                   "Br-Ma -> Ma","Br-Ma -> Br","Br-Ma -> Br-Fw-Ma","Br-Fw-Ma -> Br-Fw",
	                   "Br-Fw-Ma -> Br-Ma")
	rate       <- pset[c(1:5, 5, 4, 5,5,3,5 ,5)]
	
	dff        <- data.frame(code, code_con, transition, rate, stringsAsFactors=F)
	
	
	#     q14 : Fw    -> Br-Fw
#         q25 : Ma    -> Br-Ma
#         q34 : Br    -> Br-Fw
#         q35 : Br    -> Br-Ma
#         q41 : Br-Fw -> Fw        
#         q43 : Br-Fw -> Br                         = q41
#         q46 : Br-Fw -> Br-Fw-Ma  = (Br -> Br-Ma ) = q35
#         q52 : Br-Ma -> Ma                         = q41
#         q53 : Br-Ma -> Br                         = q41
#         q56 : Br-Ma -> Br-Fw-Ma  = (Br -> Br-Fw)  = q34
#         q64 : Br-Fw-Ma -> Br-Fw                   = q41
#         q65 : Br-Fw-Ma -> Br-Ma                   = q41 
	
	return(dff)
	 
}


optimFx <- function(pars){
	pset <- exp(pars)
	ll   <- lfx_con(pset)
	cat(ll, "\n")
	return(ll)	
}

pars_init <- rep(0.05, 5)
res <- optim(log(pars_init), fn = optimFx, method = "Nelder", control = list(fnscale = -1, maxit=2000))

partable(res$par)

tmp <-asr.marginal(lfx_con, pars = exp(res$par))

# --------- Plot tree with tip states and ancestral states
#             red: marine only, blue fw only, black , includes brackist states



# --------- Find all clades that contain only:
#                     1.  FW / Fw-Br species
#                     2.  Ma / Br-Ma species
# 
#  
 
anc_states <-asr.joint(lfx_con, pars = exp(res$par))
colvec     <- rep("black", 6)
colvec[1]  <- "blue"
colvec[2]  <- "red"

int_nodes  <- colvec[anc_states]
tip_nodes  <- colvec[statevec]

pdf(file = "salinity.pdf", height=130, width=5)

plot.phylo(v, cex=0.07, label.offset = 3, edge.width=0.4)
nodelabels(pch=19, col = int_nodes, cex=0.1)
tiplabels(pch=19, col=tip_nodes, cex = 0.1)
dev.off()



