
check_and_fix_ultrametric <- function(phy){
	
	if (!is.ultrametric(phy)){
		
		vv <- vcv.phylo(phy)
		dx <- diag(vv)
		mxx <- max(dx) - dx
		for (i in 1:length(mxx)){
			phy$edge.length[phy$edge[,2] == i] <- phy$edge.length[phy$edge[,2] == i] + mxx[i]
		}
		if (!is.ultrametric(phy)){
			stop("Ultrametric fix failed\n")
		}	
	}
	
	return(phy)
}

# ------------------------

library(ape)
library(BAMMtools)

v <- read.tree("data/actinopt_12k_treePL.tre")
v <- check_and_fix_ultrametric(v)

xx <- read.csv("data/marine-fw-rates-june2018.csv", stringsAsFactors=F)
rownames(xx) <- xx$treename

#----- State scoring
# categories
#   Fw
#   Fw-Ma
#   Ma
#   Br-Ma
#   Br-Fw
#   Br-Fw-Ma
#   Br

salinity <- character(nrow(xx))
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 0] <- "Fw"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Fw-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 0] <- "Fw-Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 0] <- "Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 0 & xx$Brackish == 1] <- "Br"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 1] <- "Br-Fw"

xx$salinity <- salinity



rfx <- function(x){
	
	rmat  <- matrix(NA, nrow=7, ncol=4)
	cvec <- c("Fw", "Br-Fw-Ma", "Fw-Ma", "Ma", "Br", "Br-Ma", "Br-Fw")
	for (i in 1:length(cvec)){
		tmp <- x[x$salinity == cvec[i], ]
		rmat[i, 1] <- nrow(tmp)
		if (nrow(tmp) > 0){
			rmat[i,2] <- mean(tmp$dr.mean)
			rmat[i,3] <- mean(tmp$tv.lambda)
			rmat[i,4] <- mean(tmp$tv.mu)
		}
	}
	
	res <- data.frame(salinity = cvec, rmat)
	colnames(res)[2:5] <- c("count", "dr", "lambda.bamm", "mu.bamm")
	return(res)
}

#------------------------------------------------------
# Extract rates and stats for each order,
#   using full dataset:
#      -- mean rates
#      -- richness
#      -- number in each category
#      -- rates for the FW and marine only species, separate.
#      -- rates overall

tx <- table(xx$order)
u_orders <- names(tx)

x   <- numeric(length(u_orders))
dff <- data.frame(order = u_orders, rich = as.numeric(tx), dr = x, lambda.bamm = x, rich_fw = x, rich_ma = x, frac_fw = x, frac_ma = x, dr_fw = x, lambda.bamm_fw = x, dr_ma = x, lambda.bamm_ma = x)

for (ii in 1:length(u_orders)){
	
	tmp <- xx[xx$order == u_orders[ii], ]
 
 	r0 <- rfx(tmp)
	r1 <- rfx(tmp[!is.na(tmp$tv.lambda), ])
	
	# percent pure FW:
	dff[ii, "frac_fw"]           <- sum(tmp$salinity == "Fw") / nrow(tmp)
	
	# percent pure Ma
	dff[ii, "frac_ma"]           <- sum(tmp$salinity == "Ma") / nrow(tmp)
	
	# mean overall rates (genetic taxa only):
	tmp2                         <- tmp[!is.na(tmp$tv.lambda), ]
	dff[ii, "dr"]                <- mean(tmp2$dr.mean)
	dff[ii, "lambda.bamm"]       <- mean(tmp2$tv.lambda)
	
	dff[ii, "rich_fw"]           <- sum(tmp$salinity == "Fw")
 	dff[ii, "rich_ma"]           <- sum(tmp$salinity == "Ma")
	dff[ii, "dr_fw"]             <- r1$dr[r1$salinity == "Fw"]
    dff[ii, "lambda.bamm_fw"]    <- r1$lambda.bamm[r1$salinity == "Fw"]
	dff[ii, "dr_ma"]             <- r1$dr[r1$salinity == "Ma"]
    dff[ii, "lambda.bamm_ma"]    <- r1$lambda.bamm[r1$salinity == "Ma"]    
    
} 
 
 
 
 
 
otophysi <- c("Gonorynchiformes", "Cypriniformes", "Characiformes", "Gymnotiformes", "Siluriformes")

tmp <- xx[xx$order %in% otophysi, ]
rfx(tmp)
rfx(xx)

rfx(xx[xx$order == "Characiformes", ])
rfx(xx[xx$order == "Gymnotiformes", ])
rfx(xx[xx$order == "Siluriformes", ])
rfx(xx[xx$order == "Cypriniformes", ])


# 3 fast clades...
oc <-  c("Gonorynchiformes", "Cypriniformes", "Characiformes", "Gymnotiformes", "Siluriformes", "Cichliformes", "Salmoniformes")
tmp <- xx[ ! xx$order %in% oc, ]
rfx(tmp)

plot.new()
par(mfrow=c(2,1))
hist(xx$dr.mean[xx$salinity == "Fw"], breaks=200, xlim=c(0,0.6))
hist(xx$dr.mean[xx$salinity == "Ma"], breaks=100, xlim=c(0,0.6))


#------------------------------
# Consider only species with genetic data:

xx <- xx[ rownames(xx) %in% v$tip.label, ]

v <- drop.tip(v, setdiff(v$tip.label, rownames(xx)))
 

#--------------------------------------
# Model of character evolution
#   
#   rate: marine to brackish
#   rate: fw to brackish
#   rate: brackish-marine to FW
#   rate: fw-brackish to marine
#   rate: loss of state (equal across all)

library(diversitree) 

xx <- xx[v$tip.label, ] 
 
statevec  <- rep(NA, nrow(xx)) 
statevec[xx$salinity == "Fw"]           <- 1
statevec[xx$salinity == "Ma"]           <- 2
statevec[xx$salinity == "Br"]           <- 3
statevec[xx$salinity == "Br-Fw"]        <- 4
statevec[xx$salinity == "Br-Ma"]        <- 5
statevec[xx$salinity == "Br-Fw-Ma"]     <- 6
 
names(statevec) <- rownames(xx)
 
lfx <- make.mkn(v, states= statevec, k=6) 

#f_constraints  <- list( q12 = 0, q13 = 0 , q14 = XXX, q15 = 0, q16 = 0,
#					    q21 = 0 , q23 = 0, q24 = 0, q25 = XXX, q26 = 0, 
#					    q31 = 0, q32 = 0, q34 = XXX, q35 = XXX, q36 = 0, 
#					    q41 = XXX, q42 = 0, q43 = XXX, q45 = 0, q46 = XXX , 
#					    q51 = 0, q52 = XXX , q53 = XXX, q54 = 0, q56 = XXX , 
#					    q61 = 0, q62 = 0, q63 = 0, q64 = XXX, q65 = XXX)
 
# pars: 
#         q14 : Fw    -> Br-Fw
#         q25 : Ma    -> Br-Ma
#         q34 : Br    -> Br-Fw
#         q35 : Br    -> Br-Ma
#         q41 : Br-Fw -> Fw        
#         q43 : Br-Fw -> Br                         = q41
#         q46 : Br-Fw -> Br-Fw-Ma  = (Br -> Br-Ma ) = q35
#         q52 : Br-Ma -> Ma                         = q41
#         q53 : Br-Ma -> Br                         = q41
#         q56 : Br-Ma -> Br-Fw-Ma  = (Br -> Br-Fw)  = q34
#         q64 : Br-Fw-Ma -> Br-Fw                   = q41
#         q65 : Br-Fw-Ma -> Br-Ma                   = q41 
 
f_constraints  <- list( q12 ~ 0, q13 ~ 0,  q15 ~ 0, q16 ~ 0,
					    q21 ~ 0 , q23 ~ 0, q24 ~ 0, q26 ~ 0, 
					    q31 ~ 0, q32 ~ 0, q36 ~ 0, 
					    q42 ~ 0, q43 ~ q41, q45 ~ 0, q46 ~ q35 , 
					    q51 ~ 0, q52 ~ q41 , q53 ~ q41, q54 ~ 0, q56 ~ q34 , 
					    q61 ~ 0, q62 ~ 0, q63 ~ 0, q64 ~ q41, q65 ~ q41)
					    
lfx_con <- constrain(lfx, formulae = f_constraints)
 
partable <- function(x){
	
	pset <- exp(x)
	#  in order: q14, q25, q34, q35, q41
	
	
	rate       <- numeric(12)
	code       <- character(12)
	code_con   <- character(12)
	transition <- character(12)
	
	code     <- c("q14", "q25", "q34", "q35", "q41", "q43", 
				"q46", "q52", "q53", "q56", "q64", "q65")
	
	code_con <- c("q14", "q25", "q34", "q35", "q41", "q41", 
				"q35", "q41", "q41", "q34", "q41", "q41")	
	
	transition <- c("Fw -> Br-Fw", "Ma -> Br-Ma", "Br -> Br-Fw", "Br -> Br-Ma",
	                 "Br-Fw -> Fw", "Br-Fw -> Br", "Br-Fw -> Br-Fw-Ma", 
	                   "Br-Ma -> Ma","Br-Ma -> Br","Br-Ma -> Br-Fw-Ma","Br-Fw-Ma -> Br-Fw",
	                   "Br-Fw-Ma -> Br-Ma")
	rate       <- pset[c(1:5, 5, 4, 5,5,3,5 ,5)]
	
	dff        <- data.frame(code, code_con, transition, rate, stringsAsFactors=F)
	
	
	#     q14 : Fw    -> Br-Fw
#         q25 : Ma    -> Br-Ma
#         q34 : Br    -> Br-Fw
#         q35 : Br    -> Br-Ma
#         q41 : Br-Fw -> Fw        
#         q43 : Br-Fw -> Br                         = q41
#         q46 : Br-Fw -> Br-Fw-Ma  = (Br -> Br-Ma ) = q35
#         q52 : Br-Ma -> Ma                         = q41
#         q53 : Br-Ma -> Br                         = q41
#         q56 : Br-Ma -> Br-Fw-Ma  = (Br -> Br-Fw)  = q34
#         q64 : Br-Fw-Ma -> Br-Fw                   = q41
#         q65 : Br-Fw-Ma -> Br-Ma                   = q41 
	
	return(dff)
	 
}


optimFx <- function(pars){
	pset <- exp(pars)
	ll   <- lfx_con(pset)
	cat(ll, "\n")
	return(ll)	
}

pars_init <- rep(0.05, 5)
res <- optim(log(pars_init), fn = optimFx, method = "Nelder", control = list(fnscale = -1, maxit=2000))

partable(res$par)

tmp <-asr.marginal(lfx_con, pars = exp(res$par))

# --------- Plot tree with tip states and ancestral states
#             red: marine only, blue fw only, black , includes brackist states



# --------- Find all clades that contain only:
#                     1.  FW / Fw-Br species
#                     2.  Ma / Br-Ma species
# 
#  
 
anc_states <-asr.joint(lfx_con, pars = exp(res$par))
colvec     <- rep("black", 6)
colvec[1]  <- "blue"
colvec[2]  <- "red"

int_nodes  <- colvec[anc_states]
tip_nodes  <- colvec[statevec]

pdf(file = "salinity.pdf", height=130, width=5)

plot.phylo(v, cex=0.07, label.offset = 3, edge.width=0.4)
nodelabels(pch=19, col = int_nodes, cex=0.1)
tiplabels(pch=19, col=tip_nodes, cex = 0.1)
dev.off()

# -------- Find all nodes involving:
#                     1. Gain of Fw, where anc includes Ma
#                     2. Gain of Fw but no brackish, anc includes Ma
#                     3. Gain of Ma, anc includes Fw
#                     4. Gain of Ma, anc with Fw, no brackish


# Testing clade extractions


node <- getMRCA(v, tip = c("Ptychocheilus_grandis", "Apteronotus_albifrons"))
xc <- extract.clade(v, node = node )
table(xx[xc$tip.label,"order"])

v$tip.label[grep("Vincigu", v$tip.label)]

node <- getMRCA(v, tip = c("Pachypanchax_sakaramyi", "Gambusia_affinis"))
xc <- extract.clade(v, node = node )
table(xx[xc$tip.label,"family"])
mean(xx[xc$tip.label, "dr.mean"])


# --------- STRAPP 

 
eventdata_vr <- paste0(basepath, 'data/event_data_thinned.csv') 
treefile     <- paste0(basepath, 'data/actinopt_12k_treePL.tre')

spdata       <- read.csv("data/marine-fw-rates-june2018.csv", stringsAsFactors=F)
rownames(spdata) <- spdata$treename

vx           <- read.tree(treefile) 
vx           <- check_and_fix_ultrametric(vx)

spdata       <- spdata[ rownames(spdata) %in% vx$tip.label, ]

# Code spdata by salinity:
salinity <- character(nrow(spdata))
salinity[spdata$Freshwater == 1 & spdata$Marine == 0 & spdata$Brackish == 0] <- "Fw"
salinity[spdata$Freshwater == 1 & spdata$Marine == 1 & spdata$Brackish == 1] <- "Br-Fw-Ma"
salinity[spdata$Freshwater == 1 & spdata$Marine == 1 & spdata$Brackish == 0] <- "Fw-Ma"
salinity[spdata$Freshwater == 0 & spdata$Marine == 1 & spdata$Brackish == 0] <- "Ma"
salinity[spdata$Freshwater == 0 & spdata$Marine == 0 & spdata$Brackish == 1] <- "Br"
salinity[spdata$Freshwater == 0 & spdata$Marine == 1 & spdata$Brackish == 1] <- "Br-Ma"
salinity[spdata$Freshwater == 1 & spdata$Marine == 0 & spdata$Brackish == 1] <- "Br-Fw"

spdata$salinity <- salinity

spdata_full <- spdata


# get set of marine-only and fw-only species:
strict       <- rownames(spdata[spdata$salinity == "Fw" | spdata$salinity == "Ma", ])
inboth       <- intersect(strict, vx$tip.label)
spdata       <- spdata[inboth, ]

# Time-varying BAMM correlation with Ma vs Fw classification
edvr        <- getEventData(vx, eventdata_vr)
edvr        <- subtreeBAMM(edvr, tips = inboth)

# Binary variable:
isFw        <- as.numeric(spdata$salinity == "Fw")
names(isFw) <- rownames(spdata)

isFw <- isFw[edvr$tip.label]

res_vr   <- traitDependentBAMM(edvr, isFw, reps = 2000, method="mann-whitney")

#-------------------------------
# FiSSE : full

source('traitDependent_functions.R')

# Use the DR rates from the full dataset:
ftree <- as.phylo(edvr)

trates <- spdata$dr.mean
names(trates) <- rownames(spdata)

res_fisse <- FISSE.binary(ftree, states = isFw, reps = 2000, tiprates = trates)

#---------------------------------
# Repeat, but remove cichlids (this is Fw vs Ma test)
spdata       <- spdata_full
strict       <- rownames(spdata[(spdata$salinity == "Fw" | spdata$salinity == "Ma") & spdata$family != "Cichlidae", ])
inboth       <- intersect(strict, vx$tip.label)
spdata       <- spdata[inboth, ]

# Time-varying BAMM correlation with Ma vs Fw classification
edvr        <- getEventData(vx, eventdata_vr)
edvr        <- subtreeBAMM(edvr, tips = inboth)

# Binary variable:
isFw        <- as.numeric(spdata$salinity == "Fw")
names(isFw) <- rownames(spdata)

isFw <- isFw[edvr$tip.label]

res_vr_nocich   <- traitDependentBAMM(edvr, isFw, reps = 2000, method="mann-whitney")

source('traitDependent_functions.R')

# Use the DR rates from the full dataset:
ftree <- as.phylo(edvr)

trates <- spdata$dr.mean
names(trates) <- rownames(spdata)

res_fisse_nocich <- FISSE.binary(ftree, states = isFw, reps = 2000, tiprates = trates)




#-------------------------------
# Repeat, but including Br-Ma and Br-Fw as states. 
spdata       <- spdata_full
strict       <- rownames(spdata[spdata$salinity %in% c("Ma", "Fw", "Br-Fw", "Br-Ma"), ])
inboth       <- intersect(strict, vx$tip.label)
spdata       <- spdata[inboth, ]

edvr        <- getEventData(vx, eventdata_vr)
edvr        <- subtreeBAMM(edvr, tips = inboth)

# Binary variable:
isFw        <- as.numeric(spdata$salinity == "Fw" | spdata$salinity == "Br-Fw")
names(isFw) <- rownames(spdata)

isFw <- isFw[edvr$tip.label]

res_vr_br   <- traitDependentBAMM(edvr, isFw, reps = 2000, method="mann-whitney")

# Use the DR rates from the full dataset:
ftree <- as.phylo(edvr)

trates <- spdata$dr.mean
names(trates) <- rownames(spdata)

res_fisse_br <- FISSE.binary(ftree, states = isFw, reps = 2000, tiprates = trates)








