library(diversitree)
library(ape)
library(BAMMtools)
source("supporting-fxns.R")

v <- read.tree("data/actinopt_12k_treePL.tre")
v <- check_and_fix_ultrametric(v)

xx <- read.csv("data/marine-fw-rates-june2018.csv", stringsAsFactors=F)
rownames(xx) <- xx$treename



 
#---- Contrasts
zz  <- read.table("data/marine-fw.txt", stringsAsFactors=F, sep="\t", header=T) 
zz  <- zz[!(is.na(zz$lambda_bamm_outgroup)) , ]
zz  <- zz[!(is.na(zz$lambda_bamm_fw))  , ]


# statistics:
t.test(x=zz$lambda_bamm_fw, y=zz$lambda_bamm_outgroup, paired=T) 
t.test(x=zz$dr_fw, y=zz$dr_outgroup, paired=T) 

wilcox.test(x=zz$dr_fw, y=zz$dr_outgroup, paired=T) 
wilcox.test(x=zz$lambda_bamm_fw, y=zz$lambda_bamm_outgroup, paired=T)  
 

#----- State scoring
# categories
#   Fw
#   Fw-Ma
#   Ma
#   Br-Ma
#   Br-Fw
#   Br-Fw-Ma
#   Br

salinity <- character(nrow(xx))
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 0] <- "Fw"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Fw-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 0] <- "Fw-Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 0] <- "Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 0 & xx$Brackish == 1] <- "Br"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 1] <- "Br-Fw"

xx$salinity <- salinity 
  
  
 
#---- Estimate speciation and extinction rates for each family separately using ML

xxg  <- xx[!is.na(xx$tv.lambda), ] # genes-only taxa
ufam <- table(xx$family)

mm   <- matrix(NA, nrow=length(ufam), ncol = 11)
colnames(mm) <- c("n_genes", "samp_frac", "Fw", "BrFwMa", "FwMa", "Ma", "Br", "BrMa", "BrFw", "lambda", "mu")

for (ii in 1:length(ufam)){
	cat(ii, "\n")
	# Number of tips with genes-only
	gtips <- sum(xxg$family == names(ufam)[ii])
	mm[ii, 1] <- gtips
	sfrac     <- as.numeric(gtips / ufam[ii])
	mm[ii, 2] <- sfrac
	
	rr <- rfx(xx[xx$family == names(ufam)[ii], ])
	
	mm[ii, 3:9] <- rr$count
	
	if (gtips > 10){
		
		vtmp <- drop.tip(v, tip = setdiff(v$tip.label, xxg$tip[xxg$family == names(ufam)[ii]]))
		
 		rx   <- fitCRBD(vtmp, sampling = sfrac)
		mm[ii, 10:11] <- rx$pars
		
		
	}
	
}

res <- data.frame(family = names(ufam), sp = as.numeric(ufam), mm, stringsAsFactors=F)
res <- res[!is.na(res$lambda), ]

res$marine_frac <- (res$Ma + res$BrMa) / res$sp
res$fw_frac     <- (res$Fw + res$BrFw) / res$sp

# define "strong-marine" or "strong-fw" has having 90% of species in either of those states.
# How many species are in these 2 categories?
sum(res$sp[res$fw_frac > 0.9 | res$marine_frac > 0.9]) / nrow(xx)   #73.0% 


plot(log10(res$sp), log10(res$lambda))

# Mean rate for strongly Fw versus strongly Ma clades:

mean(res$lambda[res$marine_frac > 0.9])
mean(res$lambda[res$fw_frac > 0.9])

# Repeat, weighting by number of tips in each group

lam_ma <- weighted.mean(res$lambda[res$marine_frac > 0.9], w = res$sp[res$marine_frac > 0.9])
lam_fw <- weighted.mean(res$lambda[res$fw_frac > 0.9], w = res$sp[res$fw_frac > 0.9])

# Test for a significant difference 
 
#----------

r2 <- res[res$marine_frac > 0.9 | res$fw_frac > 0.9, ]

quartz.options(height=6, width=10)

par(mfrow=c(1,2))
plot.new()
plot.window(xlim=c(0.9, 3.5), ylim=c(-1.75, 0.2))

cols <- rep("gray60", nrow(r2))
cols[r2$marine_frac > 0.9] <- "red"
cols[r2$fw_frac > 0.9] <- "blue"
sym <- rep(21, nrow(r2))
sym[r2$fw_frac > 0.9] <- 24

lines(x=c(0.5, 3.5), y=c(log10(c(lam_ma, lam_ma))), lwd=3, col="red", lty="dotted")
lines(x=c(0.5, 3.5), y=c(log10(c(lam_fw, lam_fw))), lwd=3, col="blue", lty="dotted")

points(log10(r2$sp), log10(r2$lambda), pch=sym, bg=cols, cex=1)
axis(1)
axis(2)

plot.new()
plot.window(xlim=c(0,2500), ylim=c(0, 1.3))
lines(x=c(0, 2500), y=c(lam_ma, lam_ma), lwd=3, col="red", lty="dotted")
lines(x=c(0, 2500), y=c(lam_fw, lam_fw), lwd=3, col="blue", lty="dotted")

points(r2$sp, r2$lambda, pch=sym, bg=cols, cex=1)

axis(1)
axis(2)

#---------------------------------------------------
# Compare 2 models: 1 where all major FW and marine clades have the same rates,
#     one where FW rates are different from marine rates.
#  Here, we use only the 138 families where:
#     -- At least 10 species are sampled in the genetic tree
#     -- At least 90% of species are either (FW, FW+BR), or (MA, MA+BR)
#


megaLFX <- list()
megaLFX$data <- r2
megaLFX$lfx <- list()
megaLFX$marine <- rep(0, nrow(r2))
megaLFX$marine[r2$marine_frac >= 0.9] <- 1
for (ii in 1:nrow(r2) ){
	
		
		vtmp <- drop.tip(v, tip = setdiff(v$tip.label, xxg$tip[xxg$family == r2$family[ii]]))
		megaLFX$lfx[[ii]] <- make.bd(vtmp, sampling.f = r2$samp_frac[ii])
	
	
}

lfx_con <- function(logpars, skip = 0){
	pars <- exp(logpars)
	ll <- 0
	for (ii in 1:length(megaLFX$lfx)){
		if (! ii %in% skip){
			ll <- ll + megaLFX$lfx[[ii]](pars)			
		}

	}		
	return(ll)
}

lfx_split <- function(logpars, skip = 0){
	pars <- exp(logpars)
	ll <- 0
	for (ii in 1:length(megaLFX$lfx)){
		if (! ii %in% skip){
			if (megaLFX$marine[ii]){
				ll <- ll + megaLFX$lfx[[ii]](pars[1:2])
			}else{
				ll <- ll + megaLFX$lfx[[ii]](pars[3:4])
			}			
		}
	}		
	return(ll)
}

lfx_sameE <- function(logpars, skip = 0){
	pars <- exp(logpars)
	ll <- 0
	for (ii in 1:length(megaLFX$lfx)){
		
		if (! ii %in% skip){
			if (megaLFX$marine[ii]){
				ll <- ll + megaLFX$lfx[[ii]](pars[c(1,3)])
			}else{
				ll <- ll + megaLFX$lfx[[ii]](pars[2:3])
			}						
		}

	}		
	return(ll)
}

lr1 <- optim(log(c(0.1,0.01)), fn = lfx_con, method = "Nelder", control = list(fnscale = -1), skip = 0)
lr2 <- optim(log(c(0.1,0.01, 0.1, 0.01)), fn = lfx_split, method = "Nelder", control = list(fnscale = -1), skip = 0)
lr3 <- optim(log(c(0.1,0.1, 0.01)), fn = lfx_sameE, method = "Nelder", control = list(fnscale = -1), skip = 0)


lr1 <- optim(log(c(0.1,0.01)), fn = lfx_con, method = "Nelder", control = list(fnscale = -1), skip = 39)
lr2 <- optim(log(c(0.1,0.01, 0.1, 0.01)), fn = lfx_split, method = "Nelder", control = list(fnscale = -1), skip = 39)
lr3 <- optim(log(c(0.1,0.1, 0.01)), fn = lfx_sameE, method = "Nelder", control = list(fnscale = -1), skip = 39)

# Or without cichlids:
omit <- which(megaLFX$data$family %in% c("Cichlidae"))

# Or without cichlids or Otophysi:
otofams <- unique(xx$family[xx$order %in% c("Siluriformes", "Characiformes", "Cypriniformes", "Gymnotiformes")])
omit <- which(megaLFX$data$family %in% c("Cichlidae", otofams))

lr1 <- optim(log(c(0.2,0.1)), fn = lfx_con, method = "Nelder", control = list(fnscale = -1), skip = omit)
lr2 <- optim(log(c(0.2,0.1, 0.2, 0.1)), fn = lfx_split, method = "Nelder", control = list(fnscale = -1), skip = omit)
lr3 <- optim(log(c(0.2,0.2, 0.1)), fn = lfx_sameE, method = "Nelder", control = list(fnscale = -1), skip = omit)

tmp <- megaLFX$data[setdiff(1:nrow(megaLFX$data), omit), ]

dropset <- which(megaLFX$data$family %in% otofams)
lr1 <- optim(log(c(0.2,0.1)), fn = lfx_con, method = "Nelder", control = list(fnscale = -1), skip = dropset)
lr2 <- optim(log(c(0.2,0.1, 0.2, 0.1)), fn = lfx_split, method = "Nelder", control = list(fnscale = -1), skip = dropset)
lr3 <- optim(log(c(0.2,0.2, 0.1)), fn = lfx_sameE, method = "Nelder", control = list(fnscale = -1), skip = dropset)

### Rate tabulations:
 

# Mean lambda for otophysi:
mean(xxg$tv.lambda[xxg$family %in% otofams])
mean(xxg$dr.mean[xxg$family %in% otofams])

# Mean lambda for cichliformes:
mean(xxg$tv.lambda[xxg$order == "Cichliformes"])
mean(xxg$dr.mean[xxg$order == "Cichliformes"])

# Mean lambda for marine
mean(xxg$tv.lambda[xxg$salinity == "Ma"])
mean(xxg$dr.mean[xxg$salinity == "Ma"])

isOtherFW <- (! xxg$family %in% otofams) & (xxg$order != "Cichliformes") & xxg$salinity == "Fw" 
 
# Mean lambda for other FW
mean(xxg$tv.lambda[isOtherFW])
mean(xxg$dr.mean[isOtherFW])


# Mean lambda for cichliformes:
mean(xxg$tv.lambda[xxg$order == "Salmoniformes"])
mean(xxg$dr.mean[xxg$order == "Salmoniformes"])








